@@ -9,23 +9,13 @@ from jax import lax
 import jax.numpy as jnp
 
 import funsor
-from numpyro.handlers import infer_config
-from numpyro.handlers import trace as OrigTraceMessenger
-from numpyro.primitives import Messenger, apply_stack
-from numpyro.primitives import plate as OrigPlateMessenger
+from numpyro.handlers import infer_config, trace as OrigTraceMessenger
+from numpyro.primitives import Messenger, apply_stack, plate as OrigPlateMessenger
 
 funsor.set_backend("jax")
 
 
-__all__ = [
-    "enum",
-    "infer_config",
-    "markov",
-    "plate",
-    "to_data",
-    "to_funsor",
-    "trace",
-]
+__all__ = ["enum", "infer_config", "markov", "plate", "to_data", "to_funsor", "trace"]
 
 
 ##################################
@@ -33,14 +23,11 @@ __all__ = [
 ##################################
 
 # name_to_dim : dict, dim_to_name : dict, parents : tuple, iter_parents : tuple
-class StackFrame(namedtuple('StackFrame', [
-            'name_to_dim',
-            'dim_to_name',
-            'parents',
-            'iter_parents',
-            'keep',
-        ])):
-
+class StackFrame(
+    namedtuple(
+        "StackFrame", ["name_to_dim", "dim_to_name", "parents", "iter_parents", "keep"]
+    )
+):
     def read(self, name, dim):
         found_name = self.dim_to_name.get(dim, name)
         found_dim = self.name_to_dim.get(name, dim)
@@ -60,14 +47,15 @@ class StackFrame(namedtuple('StackFrame', [
 
 class DimType(Enum):
     """Enumerates the possible types of dimensions to allocate"""
+
     LOCAL = 0
     GLOBAL = 1
     VISIBLE = 2
 
 
-DimRequest = namedtuple('DimRequest', ['dim', 'dim_type'])
+DimRequest = namedtuple("DimRequest", ["dim", "dim_type"])
 DimRequest.__new__.__defaults__ = (None, DimType.LOCAL)
-NameRequest = namedtuple('NameRequest', ['name', 'dim_type'])
+NameRequest = namedtuple("NameRequest", ["name", "dim_type"])
 NameRequest.__new__.__defaults__ = (None, DimType.LOCAL)
 
 
@@ -78,11 +66,17 @@ class DimStack:
     Replaces the plate DimAllocator, the enum EnumAllocator, the stack in MarkovMessenger,
     _param_dims and _value_dims in EnumMessenger, and dim_to_symbol in msg['infer']
     """
+
     def __init__(self):
-        self._stack = [StackFrame(
-            name_to_dim=OrderedDict(), dim_to_name=OrderedDict(),
-            parents=(), iter_parents=(), keep=False,
-        )]
+        self._stack = [
+            StackFrame(
+                name_to_dim=OrderedDict(),
+                dim_to_name=OrderedDict(),
+                parents=(),
+                iter_parents=(),
+                keep=False,
+            )
+        ]
         self._first_available_dim = -1
         self.outermost = None
 
@@ -109,7 +103,9 @@ class DimStack:
         return self._stack[0]
 
     def _gendim(self, name_request, dim_request):
-        assert isinstance(name_request, NameRequest) and isinstance(dim_request, DimRequest)
+        assert isinstance(name_request, NameRequest) and isinstance(
+            dim_request, DimRequest
+        )
         dim_type = dim_request.dim_type
 
         if name_request.name is None:
@@ -117,8 +113,11 @@ class DimStack:
         else:
             fresh_name = name_request.name
 
-        conflict_frames = (self.current_frame, self.global_frame) + \
-            self.current_frame.parents + self.current_frame.iter_parents
+        conflict_frames = (
+            (self.current_frame, self.global_frame)
+            + self.current_frame.parents
+            + self.current_frame.iter_parents
+        )
         if dim_request.dim is None:
             fresh_dim = self._first_available_dim if dim_type != DimType.VISIBLE else -1
             fresh_dim = -1 if fresh_dim is None else fresh_dim
@@ -127,9 +126,11 @@ class DimStack:
         else:
             fresh_dim = dim_request.dim
 
-        if fresh_dim < self.MAX_DIM or \
-                any(fresh_dim in p.dim_to_name for p in conflict_frames) or \
-                (dim_type == DimType.VISIBLE and fresh_dim <= self._first_available_dim):
+        if (
+            fresh_dim < self.MAX_DIM
+            or any(fresh_dim in p.dim_to_name for p in conflict_frames)
+            or (dim_type == DimType.VISIBLE and fresh_dim <= self._first_available_dim)
+        ):
             raise ValueError(f"Ran out of free dims during allocation for {fresh_name}")
 
         return fresh_name, fresh_dim
@@ -141,8 +142,14 @@ class DimStack:
         elif isinstance(name, NameRequest):
             name, dim_type = name.name, name.dim_type
 
-        read_frames = (self.global_frame,) if dim_type != DimType.LOCAL else \
-            (self.current_frame,) + self.current_frame.parents + self.current_frame.iter_parents + (self.global_frame,)
+        read_frames = (
+            (self.global_frame,)
+            if dim_type != DimType.LOCAL
+            else (self.current_frame,)
+            + self.current_frame.parents
+            + self.current_frame.iter_parents
+            + (self.global_frame,)
+        )
 
         # read dimension
         for frame in read_frames:
@@ -152,10 +159,16 @@ class DimStack:
 
         # generate fresh name or dimension
         if not found:
-            name, dim = self._gendim(NameRequest(name, dim_type), DimRequest(dim, dim_type))
+            name, dim = self._gendim(
+                NameRequest(name, dim_type), DimRequest(dim, dim_type)
+            )
 
-            write_frames = (self.global_frame,) if dim_type != DimType.LOCAL else \
-                (self.current_frame,) + (self.current_frame.parents if self.current_frame.keep else ())
+            write_frames = (
+                (self.global_frame,)
+                if dim_type != DimType.LOCAL
+                else (self.current_frame,)
+                + (self.current_frame.parents if self.current_frame.keep else ())
+            )
 
             # store the fresh dimension
             for frame in write_frames:
@@ -171,6 +184,7 @@ _DIM_STACK = DimStack()  # only one global instance
 # Messengers that implement guts of enumeration
 #################################################
 
+
 class ReentrantMessenger(Messenger):
     def __init__(self, fn=None):
         self._ref_count = 0
@@ -192,7 +206,6 @@ class ReentrantMessenger(Messenger):
 
 
 class DimStackCleanupMessenger(ReentrantMessenger):
-
     def __init__(self, fn=None):
         self._saved_dims = ()
         return super().__init__(fn)
@@ -208,13 +221,14 @@ class DimStackCleanupMessenger(ReentrantMessenger):
     def __exit__(self, *args, **kwargs):
         if self._ref_count == 1 and _DIM_STACK.outermost is self:
             _DIM_STACK.outermost = None
-            for name, dim in reversed(tuple(_DIM_STACK.global_frame.name_to_dim.items())):
+            for name, dim in reversed(
+                tuple(_DIM_STACK.global_frame.name_to_dim.items())
+            ):
                 self._saved_dims += (_DIM_STACK.global_frame.free(name, dim),)
         return super().__exit__(*args, **kwargs)
 
 
 class NamedMessenger(DimStackCleanupMessenger):
-
     def process_message(self, msg):
         if msg["type"] == "to_funsor":
             self._pyro_to_funsor(msg)
@@ -228,7 +242,9 @@ class NamedMessenger(DimStackCleanupMessenger):
         # interpret all names/dims as requests since we only run this function once
         for name in batch_names:
             dim = name_to_dim.get(name, None)
-            name_to_dim[name] = dim if isinstance(dim, DimRequest) else DimRequest(dim, dim_type)
+            name_to_dim[name] = (
+                dim if isinstance(dim, DimRequest) else DimRequest(dim, dim_type)
+            )
 
         # read dimensions and allocate fresh dimensions as necessary
         for name, dim_request in name_to_dim.items():
@@ -239,13 +255,17 @@ class NamedMessenger(DimStackCleanupMessenger):
     @classmethod  # only depends on the global _DIM_STACK state, not self
     def _pyro_to_data(cls, msg):
 
-        funsor_value, = msg["args"]
+        (funsor_value,) = msg["args"]
         name_to_dim = msg["kwargs"].setdefault("name_to_dim", OrderedDict())
         dim_type = msg["kwargs"].setdefault("dim_type", DimType.LOCAL)
 
         batch_names = tuple(funsor_value.inputs.keys())
 
-        name_to_dim.update(cls._get_name_to_dim(batch_names, name_to_dim=name_to_dim, dim_type=dim_type))
+        name_to_dim.update(
+            cls._get_name_to_dim(
+                batch_names, name_to_dim=name_to_dim, dim_type=dim_type
+            )
+        )
         msg["stop"] = True  # only need to run this once per to_data call
 
     @staticmethod
@@ -260,7 +280,9 @@ class NamedMessenger(DimStackCleanupMessenger):
             # before they have been assigned a name
             if batch_shape[dim] == 1 and name is None:
                 continue
-            dim_to_name[dim] = name if isinstance(name, NameRequest) else NameRequest(name, dim_type)
+            dim_to_name[dim] = (
+                name if isinstance(name, NameRequest) else NameRequest(name, dim_type)
+            )
 
         for dim, name_request in dim_to_name.items():
             dim_to_name[dim] = _DIM_STACK.request(name_request, dim)[0]
@@ -282,9 +304,13 @@ class NamedMessenger(DimStackCleanupMessenger):
         try:
             batch_shape = raw_value.batch_shape  # TODO make make this more robust
         except AttributeError:
-            batch_shape = raw_value.shape[:len(raw_value.shape) - event_dim]
+            batch_shape = raw_value.shape[: len(raw_value.shape) - event_dim]
 
-        dim_to_name.update(cls._get_dim_to_name(batch_shape, dim_to_name=dim_to_name, dim_type=dim_type))
+        dim_to_name.update(
+            cls._get_dim_to_name(
+                batch_shape, dim_to_name=dim_to_name, dim_type=dim_type
+            )
+        )
         msg["stop"] = True  # only need to run this once per to_funsor call
 
 
@@ -300,6 +326,7 @@ class LocalNamedMessenger(NamedMessenger):
         level can depend on each other; if ``keep=False``, neighboring branches
         are independent (conditioned on their shared ancestors).
     """
+
     def __init__(self, fn=None, history=1, keep=False):
         self.history = history
         self.keep = keep
@@ -336,10 +363,13 @@ class LocalNamedMessenger(NamedMessenger):
             name_to_dim, dim_to_name = OrderedDict(), OrderedDict()
 
         frame = StackFrame(
-            name_to_dim=name_to_dim, dim_to_name=dim_to_name,
-            parents=tuple(reversed(_DIM_STACK._stack[len(_DIM_STACK._stack) - self.history:])),
+            name_to_dim=name_to_dim,
+            dim_to_name=dim_to_name,
+            parents=tuple(
+                reversed(_DIM_STACK._stack[len(_DIM_STACK._stack) - self.history :])
+            ),
             iter_parents=tuple(self._iter_parents),
-            keep=self.keep
+            keep=self.keep,
         )
 
         _DIM_STACK.push(frame)
@@ -350,8 +380,11 @@ class LocalNamedMessenger(NamedMessenger):
             # don't keep around references to other frames
             old_frame = _DIM_STACK.pop()
             saved_frame = StackFrame(
-                name_to_dim=old_frame.name_to_dim, dim_to_name=old_frame.dim_to_name,
-                parents=(), iter_parents=(), keep=self.keep
+                name_to_dim=old_frame.name_to_dim,
+                dim_to_name=old_frame.dim_to_name,
+                parents=(),
+                iter_parents=(),
+                keep=self.keep,
             )
             self._saved_frames.append(saved_frame)
         else:
@@ -360,7 +393,6 @@ class LocalNamedMessenger(NamedMessenger):
 
 
 class GlobalNamedMessenger(NamedMessenger):
-
     def __init__(self, fn=None):
         self._saved_globals = ()
         super().__init__(fn)
@@ -387,26 +419,35 @@ class GlobalNamedMessenger(NamedMessenger):
     def _pyro_post_to_funsor(self, msg):
         if msg["kwargs"]["dim_type"] in (DimType.GLOBAL, DimType.VISIBLE):
             for name in msg["value"].inputs:
-                self._saved_globals += ((name, _DIM_STACK.global_frame.name_to_dim[name]),)
+                self._saved_globals += (
+                    (name, _DIM_STACK.global_frame.name_to_dim[name]),
+                )
 
     def _pyro_post_to_data(self, msg):
         if msg["kwargs"]["dim_type"] in (DimType.GLOBAL, DimType.VISIBLE):
             for name in msg["args"][0].inputs:
-                self._saved_globals += ((name, _DIM_STACK.global_frame.name_to_dim[name]),)
+                self._saved_globals += (
+                    (name, _DIM_STACK.global_frame.name_to_dim[name]),
+                )
 
 
 class BaseEnumMessenger(NamedMessenger):
     """
     Handles first_available_dim management, enum effects should inherit from this
     """
+
     def __init__(self, fn=None, first_available_dim=None):
-        assert first_available_dim is None or first_available_dim < 0, first_available_dim
+        assert (
+            first_available_dim is None or first_available_dim < 0
+        ), first_available_dim
         self.first_available_dim = first_available_dim
         super().__init__(fn)
 
     def __enter__(self):
         if self._ref_count == 0 and self.first_available_dim is not None:
-            self._prev_first_dim = _DIM_STACK.set_first_available_dim(self.first_available_dim)
+            self._prev_first_dim = _DIM_STACK.set_first_available_dim(
+                self.first_available_dim
+            )
         return super().__enter__()
 
     def __exit__(self, *args, **kwargs):
@@ -419,6 +460,7 @@ class BaseEnumMessenger(NamedMessenger):
 # User-facing handler implementations
 ##########################################
 
+
 class plate(GlobalNamedMessenger):
     """
     An alternative implementation of :class:`numpyro.primitives.plate` primitive. Note
@@ -437,24 +479,33 @@ class plate(GlobalNamedMessenger):
         is used as the plate dim. If `None` (default), the leftmost available dim
         is allocated.
     """
+
     def __init__(self, name, size, subsample_size=None, dim=None):
         self.name = name
         self.size = size
         if dim is not None and dim >= 0:
-            raise ValueError('dim arg must be negative.')
-        self.dim, indices = OrigPlateMessenger._subsample(self.name, self.size, subsample_size, dim)
+            raise ValueError("dim arg must be negative.")
+        self.dim, indices = OrigPlateMessenger._subsample(
+            self.name, self.size, subsample_size, dim
+        )
         self.subsample_size = indices.shape[0]
         self._indices = funsor.Tensor(
             indices,
-            OrderedDict([(self.name, funsor.bint(self.subsample_size))]),
-            self.subsample_size
+            OrderedDict([(self.name, funsor.Bint[self.subsample_size])]),
+            self.subsample_size,
         )
         super(plate, self).__init__(None)
 
     def __enter__(self):
         super().__enter__()  # do this first to take care of globals recycling
-        name_to_dim = OrderedDict([(self.name, self.dim)]) if self.dim is not None else OrderedDict()
-        indices = to_data(self._indices, name_to_dim=name_to_dim, dim_type=DimType.VISIBLE)
+        name_to_dim = (
+            OrderedDict([(self.name, self.dim)])
+            if self.dim is not None
+            else OrderedDict()
+        )
+        indices = to_data(
+            self._indices, name_to_dim=name_to_dim, dim_type=DimType.VISIBLE
+        )
         # extract the dimension allocated by to_data to match plate's current behavior
         self.dim, self.indices = -len(indices.shape), indices.squeeze()
         return self.indices
@@ -486,12 +537,18 @@ class plate(GlobalNamedMessenger):
                 if len(shape) >= -dim and shape[dim] != 1:
                     if shape[dim] != self.size:
                         if msg["type"] == "param":
-                            statement = "numpyro.param({}, ..., event_dim={})".format(msg["name"], event_dim)
+                            statement = "numpyro.param({}, ..., event_dim={})".format(
+                                msg["name"], event_dim
+                            )
                         else:
-                            statement = "numpyro.subsample(..., event_dim={})".format(event_dim)
+                            statement = "numpyro.subsample(..., event_dim={})".format(
+                                event_dim
+                            )
                         raise ValueError(
-                            "Inside numpyro.plate({}, {}, dim={}) invalid shape of {}: {}"
-                            .format(self.name, self.size, self.dim, statement, shape))
+                            "Inside numpyro.plate({}, {}, dim={}) invalid shape of {}: {}".format(
+                                self.name, self.size, self.dim, statement, shape
+                            )
+                        )
                     if self.subsample_size < self.size:
                         value = msg["value"]
                         new_value = jnp.take(value, self.indices, dim)
@@ -509,10 +566,16 @@ class enum(BaseEnumMessenger):
         dimension and all dimensions left may be used internally by Pyro.
         This should be a negative integer or None.
     """
+
     def process_message(self, msg):
-        if msg["type"] != "sample" or \
-                msg.get("done", False) or msg["is_observed"] or msg["infer"].get("expand", False) or \
-                msg["infer"].get("enumerate") != "parallel" or (not msg["fn"].has_enumerate_support):
+        if (
+            msg["type"] != "sample"
+            or msg.get("done", False)
+            or msg["is_observed"]
+            or msg["infer"].get("expand", False)
+            or msg["infer"].get("enumerate") != "parallel"
+            or (not msg["fn"].has_enumerate_support)
+        ):
             if msg["type"] == "control_flow":
                 msg["kwargs"]["enum"] = True
                 msg["kwargs"]["first_available_dim"] = self.first_available_dim
@@ -527,9 +590,7 @@ class enum(BaseEnumMessenger):
         size = msg["fn"].enumerate_support(expand=False).shape[0]
         raw_value = jnp.arange(0, size)
         funsor_value = funsor.Tensor(
-            raw_value,
-            OrderedDict([(msg["name"], funsor.bint(size))]),
-            size
+            raw_value, OrderedDict([(msg["name"], funsor.Bint[size])]), size
         )
 
         msg["value"] = to_data(funsor_value)
@@ -544,13 +605,19 @@ class trace(OrigTraceMessenger):
     Each sample site is annotated with a "dim_to_name" dictionary,
     which can be passed directly to :func:`to_funsor`.
     """
+
     def postprocess_message(self, msg):
         if msg["type"] == "sample":
             total_batch_shape = lax.broadcast_shapes(
                 tuple(msg["fn"].batch_shape),
-                jnp.shape(msg["value"])[:jnp.ndim(msg["value"]) - msg["fn"].event_dim]
+                jnp.shape(msg["value"])[: jnp.ndim(msg["value"]) - msg["fn"].event_dim],
             )
-            msg["infer"]["dim_to_name"] = NamedMessenger._get_dim_to_name(total_batch_shape)
+            msg["infer"]["dim_to_name"] = NamedMessenger._get_dim_to_name(
+                total_batch_shape
+            )
+            msg["infer"]["name_to_dim"] = {
+                name: dim for dim, name in msg["infer"]["dim_to_name"].items()
+            }
         if msg["type"] in ("sample", "param"):
             super().postprocess_message(msg)
 
@@ -571,7 +638,9 @@ def markov(fn=None, history=1, keep=False):
         are independent (conditioned on their shared ancestors).
     """
     if fn is not None and not callable(fn):  # Used as a generator
-        return LocalNamedMessenger(fn=None, history=history, keep=keep).generator(iterable=fn)
+        return LocalNamedMessenger(fn=None, history=history, keep=keep).generator(
+            iterable=fn
+        )
     return LocalNamedMessenger(fn, history=history, keep=keep)
 
 
@@ -579,6 +648,7 @@ def markov(fn=None, history=1, keep=False):
 # New primitives
 ####################
 
+
 def to_funsor(x, output=None, dim_to_name=None, dim_type=DimType.LOCAL):
     """
     A primitive to convert a Python object to a :class:`~funsor.terms.Funsor`.
@@ -597,16 +667,18 @@ def to_funsor(x, output=None, dim_to_name=None, dim_type=DimType.LOCAL):
     dim_to_name = OrderedDict() if dim_to_name is None else dim_to_name
 
     initial_msg = {
-        'type': 'to_funsor',
-        'fn': lambda x, output, dim_to_name, dim_type: funsor.to_funsor(x, output=output, dim_to_name=dim_to_name),
-        'args': (x,),
-        'kwargs': {"output": output, "dim_to_name": dim_to_name, "dim_type": dim_type},
-        'value': None,
-        'mask': None,
+        "type": "to_funsor",
+        "fn": lambda x, output, dim_to_name, dim_type: funsor.to_funsor(
+            x, output=output, dim_to_name=dim_to_name
+        ),
+        "args": (x,),
+        "kwargs": {"output": output, "dim_to_name": dim_to_name, "dim_type": dim_type},
+        "value": None,
+        "mask": None,
     }
 
     msg = apply_stack(initial_msg)
-    return msg['value']
+    return msg["value"]
 
 
 def to_data(x, name_to_dim=None, dim_type=DimType.LOCAL):
@@ -624,13 +696,15 @@ def to_data(x, name_to_dim=None, dim_type=DimType.LOCAL):
     name_to_dim = OrderedDict() if name_to_dim is None else name_to_dim
 
     initial_msg = {
-        'type': 'to_data',
-        'fn': lambda x, name_to_dim, dim_type: funsor.to_data(x, name_to_dim=name_to_dim),
-        'args': (x,),
-        'kwargs': {"name_to_dim": name_to_dim, "dim_type": dim_type},
-        'value': None,
-        'mask': None,
+        "type": "to_data",
+        "fn": lambda x, name_to_dim, dim_type: funsor.to_data(
+            x, name_to_dim=name_to_dim
+        ),
+        "args": (x,),
+        "kwargs": {"name_to_dim": name_to_dim, "dim_type": dim_type},
+        "value": None,
+        "mask": None,
     }
 
     msg = apply_stack(initial_msg)
-    return msg['value']
+    return msg["value"]
