@@ -1,13 +1,18 @@
 # Copyright Contributors to the Pyro project.
 # SPDX-License-Identifier: Apache-2.0
 
-from jax import lax, random
+from jax import lax, nn, random
 import jax.numpy as jnp
-from jax.scipy.special import betaln, gammaln
+from jax.scipy.special import betainc, betaln, gammaln
 
 from numpyro.distributions import constraints
 from numpyro.distributions.continuous import Beta, Dirichlet, Gamma
-from numpyro.distributions.discrete import BinomialProbs, MultinomialProbs, Poisson
+from numpyro.distributions.discrete import (
+    BinomialProbs,
+    MultinomialProbs,
+    Poisson,
+    ZeroInflatedDistribution,
+)
 from numpyro.distributions.distribution import Distribution
 from numpyro.distributions.util import is_prng_key, promote_shapes, validate_sample
 
@@ -30,18 +35,23 @@ class BetaBinomial(Distribution):
         Beta distribution.
     :param numpy.ndarray total_count: number of Bernoulli trials.
     """
-    arg_constraints = {'concentration1': constraints.positive, 'concentration0': constraints.positive,
-                       'total_count': constraints.nonnegative_integer}
+    arg_constraints = {
+        "concentration1": constraints.positive,
+        "concentration0": constraints.positive,
+        "total_count": constraints.nonnegative_integer,
+    }
     has_enumerate_support = True
-    is_discrete = True
     enumerate_support = BinomialProbs.enumerate_support
 
-    def __init__(self, concentration1, concentration0, total_count=1, validate_args=None):
+    def __init__(
+        self, concentration1, concentration0, total_count=1, validate_args=None
+    ):
         self.concentration1, self.concentration0, self.total_count = promote_shapes(
             concentration1, concentration0, total_count
         )
-        batch_shape = lax.broadcast_shapes(jnp.shape(concentration1), jnp.shape(concentration0),
-                                           jnp.shape(total_count))
+        batch_shape = lax.broadcast_shapes(
+            jnp.shape(concentration1), jnp.shape(concentration0), jnp.shape(total_count)
+        )
         concentration1 = jnp.broadcast_to(concentration1, batch_shape)
         concentration0 = jnp.broadcast_to(concentration0, batch_shape)
         self._beta = Beta(concentration1, concentration0)
@@ -51,13 +61,20 @@ class BetaBinomial(Distribution):
         assert is_prng_key(key)
         key_beta, key_binom = random.split(key)
         probs = self._beta.sample(key_beta, sample_shape)
-        return BinomialProbs(total_count=self.total_count, probs=probs).sample(key_binom)
+        return BinomialProbs(total_count=self.total_count, probs=probs).sample(
+            key_binom
+        )
 
     @validate_sample
     def log_prob(self, value):
-        return (-_log_beta_1(self.total_count - value + 1, value) +
-                betaln(value + self.concentration1, self.total_count - value + self.concentration0) -
-                betaln(self.concentration0, self.concentration1))
+        return (
+            -_log_beta_1(self.total_count - value + 1, value)
+            + betaln(
+                value + self.concentration1,
+                self.total_count - value + self.concentration0,
+            )
+            - betaln(self.concentration0, self.concentration1)
+        )
 
     @property
     def mean(self):
@@ -65,9 +82,13 @@ class BetaBinomial(Distribution):
 
     @property
     def variance(self):
-        return self._beta.variance * self.total_count * (self.concentration0 + self.concentration1 + self.total_count)
+        return (
+            self._beta.variance
+            * self.total_count
+            * (self.concentration0 + self.concentration1 + self.total_count)
+        )
 
-    @property
+    @constraints.dependent_property(is_discrete=True, event_dim=0)
     def support(self):
         return constraints.integer_interval(0, self.total_count)
 
@@ -84,34 +105,45 @@ class DirichletMultinomial(Distribution):
         Dirichlet distribution.
     :param numpy.ndarray total_count: number of Categorical trials.
     """
-    arg_constraints = {'concentration': constraints.positive,
-                       'total_count': constraints.nonnegative_integer}
-    is_discrete = True
+    arg_constraints = {
+        "concentration": constraints.independent(constraints.positive, 1),
+        "total_count": constraints.nonnegative_integer,
+    }
 
     def __init__(self, concentration, total_count=1, validate_args=None):
         if jnp.ndim(concentration) < 1:
-            raise ValueError("`concentration` parameter must be at least one-dimensional.")
+            raise ValueError(
+                "`concentration` parameter must be at least one-dimensional."
+            )
 
-        batch_shape = lax.broadcast_shapes(jnp.shape(concentration)[:-1], jnp.shape(total_count))
+        batch_shape = lax.broadcast_shapes(
+            jnp.shape(concentration)[:-1], jnp.shape(total_count)
+        )
         concentration_shape = batch_shape + jnp.shape(concentration)[-1:]
-        self.concentration, = promote_shapes(concentration, shape=concentration_shape)
-        self.total_count, = promote_shapes(total_count, shape=batch_shape)
+        (self.concentration,) = promote_shapes(concentration, shape=concentration_shape)
+        (self.total_count,) = promote_shapes(total_count, shape=batch_shape)
         concentration = jnp.broadcast_to(self.concentration, concentration_shape)
         self._dirichlet = Dirichlet(concentration)
         super().__init__(
-            self._dirichlet.batch_shape, self._dirichlet.event_shape, validate_args=validate_args)
+            self._dirichlet.batch_shape,
+            self._dirichlet.event_shape,
+            validate_args=validate_args,
+        )
 
     def sample(self, key, sample_shape=()):
         assert is_prng_key(key)
         key_dirichlet, key_multinom = random.split(key)
         probs = self._dirichlet.sample(key_dirichlet, sample_shape)
-        return MultinomialProbs(total_count=self.total_count, probs=probs).sample(key_multinom)
+        return MultinomialProbs(total_count=self.total_count, probs=probs).sample(
+            key_multinom
+        )
 
     @validate_sample
     def log_prob(self, value):
         alpha = self.concentration
-        return (_log_beta_1(alpha.sum(-1), value.sum(-1)) -
-                _log_beta_1(alpha, value).sum(-1))
+        return _log_beta_1(alpha.sum(-1), value.sum(-1)) - _log_beta_1(
+            alpha, value
+        ).sum(-1)
 
     @property
     def mean(self):
@@ -125,10 +157,16 @@ class DirichletMultinomial(Distribution):
         alpha_ratio = alpha / alpha_sum
         return n * alpha_ratio * (1 - alpha_ratio) * (n + alpha_sum) / (1 + alpha_sum)
 
-    @property
+    @constraints.dependent_property(is_discrete=True, event_dim=1)
     def support(self):
         return constraints.multinomial(self.total_count)
 
+    @staticmethod
+    def infer_shapes(concentration, total_count=()):
+        batch_shape = lax.broadcast_shapes(concentration[:-1], total_count)
+        event_shape = concentration[-1:]
+        return batch_shape, event_shape
+
 
 class GammaPoisson(Distribution):
     r"""
@@ -140,14 +178,18 @@ class GammaPoisson(Distribution):
     :param numpy.ndarray concentration: shape parameter (alpha) of the Gamma distribution.
     :param numpy.ndarray rate: rate parameter (beta) for the Gamma distribution.
     """
-    arg_constraints = {'concentration': constraints.positive, 'rate': constraints.positive}
+    arg_constraints = {
+        "concentration": constraints.positive,
+        "rate": constraints.positive,
+    }
     support = constraints.nonnegative_integer
-    is_discrete = True
 
-    def __init__(self, concentration, rate=1., validate_args=None):
+    def __init__(self, concentration, rate=1.0, validate_args=None):
         self.concentration, self.rate = promote_shapes(concentration, rate)
         self._gamma = Gamma(concentration, rate)
-        super(GammaPoisson, self).__init__(self._gamma.batch_shape, validate_args=validate_args)
+        super(GammaPoisson, self).__init__(
+            self._gamma.batch_shape, validate_args=validate_args
+        )
 
     def sample(self, key, sample_shape=()):
         assert is_prng_key(key)
@@ -158,8 +200,12 @@ class GammaPoisson(Distribution):
     @validate_sample
     def log_prob(self, value):
         post_value = self.concentration + value
-        return -betaln(self.concentration, value + 1) - jnp.log(post_value) + \
-            self.concentration * jnp.log(self.rate) - post_value * jnp.log1p(self.rate)
+        return (
+            -betaln(self.concentration, value + 1)
+            - jnp.log(post_value)
+            + self.concentration * jnp.log(self.rate)
+            - post_value * jnp.log1p(self.rate)
+        )
 
     @property
     def mean(self):
@@ -168,3 +214,79 @@ class GammaPoisson(Distribution):
     @property
     def variance(self):
         return self.concentration / jnp.square(self.rate) * (1 + self.rate)
+
+    def cdf(self, value):
+        bt = betainc(self.concentration, value + 1.0, self.rate / (self.rate + 1.0))
+        return bt
+
+
+def NegativeBinomial(total_count, probs=None, logits=None, validate_args=None):
+    if probs is not None:
+        return NegativeBinomialProbs(total_count, probs, validate_args=validate_args)
+    elif logits is not None:
+        return NegativeBinomialLogits(total_count, logits, validate_args=validate_args)
+    else:
+        raise ValueError("One of `probs` or `logits` must be specified.")
+
+
+class NegativeBinomialProbs(GammaPoisson):
+    arg_constraints = {
+        "total_count": constraints.positive,
+        "probs": constraints.unit_interval,
+    }
+    support = constraints.nonnegative_integer
+
+    def __init__(self, total_count, probs, validate_args=None):
+        self.total_count, self.probs = promote_shapes(total_count, probs)
+        concentration = total_count
+        rate = 1.0 / probs - 1.0
+        super().__init__(concentration, rate, validate_args=validate_args)
+
+
+class NegativeBinomialLogits(GammaPoisson):
+    arg_constraints = {
+        "total_count": constraints.positive,
+        "logits": constraints.real,
+    }
+    support = constraints.nonnegative_integer
+
+    def __init__(self, total_count, logits, validate_args=None):
+        self.total_count, self.logits = promote_shapes(total_count, logits)
+        concentration = total_count
+        rate = jnp.exp(-logits)
+        super().__init__(concentration, rate, validate_args=validate_args)
+
+    @validate_sample
+    def log_prob(self, value):
+        return -(
+            self.total_count * nn.softplus(self.logits)
+            + value * nn.softplus(-self.logits)
+            + _log_beta_1(self.total_count, value)
+        )
+
+
+class NegativeBinomial2(GammaPoisson):
+    """
+    Another parameterization of GammaPoisson with `rate` is replaced by `mean`.
+    """
+
+    arg_constraints = {
+        "mean": constraints.positive,
+        "concentration": constraints.positive,
+    }
+    support = constraints.nonnegative_integer
+
+    def __init__(self, mean, concentration, validate_args=None):
+        rate = concentration / mean
+        super().__init__(concentration, rate, validate_args=validate_args)
+
+
+def ZeroInflatedNegativeBinomial2(
+    mean, concentration, *, gate=None, gate_logits=None, validate_args=None
+):
+    return ZeroInflatedDistribution(
+        NegativeBinomial2(mean, concentration, validate_args=validate_args),
+        gate=gate,
+        gate_logits=gate_logits,
+        validate_args=validate_args,
+    )
