@@ -27,27 +27,30 @@
 
 
 __all__ = [
-    'boolean',
-    'corr_cholesky',
-    'corr_matrix',
-    'dependent',
-    'greater_than',
-    'integer_interval',
-    'integer_greater_than',
-    'interval',
-    'is_dependent',
-    'less_than',
-    'lower_cholesky',
-    'multinomial',
-    'nonnegative_integer',
-    'positive',
-    'positive_definite',
-    'positive_integer',
-    'real',
-    'real_vector',
-    'simplex',
-    'unit_interval',
-    'Constraint',
+    "boolean",
+    "corr_cholesky",
+    "corr_matrix",
+    "dependent",
+    "greater_than",
+    "integer_interval",
+    "integer_greater_than",
+    "interval",
+    "is_dependent",
+    "less_than",
+    "lower_cholesky",
+    "multinomial",
+    "nonnegative_integer",
+    "positive",
+    "positive_definite",
+    "positive_integer",
+    "real",
+    "real_vector",
+    "simplex",
+    "sphere",
+    "softplus_lower_cholesky",
+    "softplus_positive",
+    "unit_interval",
+    "Constraint",
 ]
 
 import numpy as np
@@ -62,6 +65,8 @@ class Constraint(object):
     A constraint object represents a region over which a variable is valid,
     e.g. within which a variable can be optimized.
     """
+
+    is_discrete = False
     event_dim = 0
 
     def __call__(self, x):
@@ -82,6 +87,8 @@ class Constraint(object):
 
 
 class _Boolean(Constraint):
+    is_discrete = True
+
     def __call__(self, x):
         return (x == 0) | (x == 1)
 
@@ -95,14 +102,18 @@ class _CorrCholesky(Constraint):
     def __call__(self, x):
         jnp = np if isinstance(x, (np.ndarray, np.generic)) else jax.numpy
         tril = jnp.tril(x)
-        lower_triangular = jnp.all(jnp.reshape(tril == x, x.shape[:-2] + (-1,)), axis=-1)
+        lower_triangular = jnp.all(
+            jnp.reshape(tril == x, x.shape[:-2] + (-1,)), axis=-1
+        )
         positive_diagonal = jnp.all(jnp.diagonal(x, axis1=-2, axis2=-1) > 0, axis=-1)
         x_norm = jnp.linalg.norm(x, axis=-1)
         unit_norm_row = jnp.all((x_norm <= 1) & (x_norm > 1 - 1e-6), axis=-1)
         return lower_triangular & positive_diagonal & unit_norm_row
 
     def feasible_like(self, prototype):
-        return jax.numpy.broadcast_to(jax.numpy.eye(prototype.shape[-1]), prototype.shape)
+        return jax.numpy.broadcast_to(
+            jax.numpy.eye(prototype.shape[-1]), prototype.shape
+        )
 
 
 class _CorrMatrix(Constraint):
@@ -115,16 +126,79 @@ class _CorrMatrix(Constraint):
         # check for the smallest eigenvalue is positive
         positive = jnp.linalg.eigh(x)[0][..., 0] > 0
         # check for diagonal equal to 1
-        unit_variance = jnp.all(jnp.abs(jnp.diagonal(x, axis1=-2, axis2=-1) - 1) < 1e-6, axis=-1)
+        unit_variance = jnp.all(
+            jnp.abs(jnp.diagonal(x, axis1=-2, axis2=-1) - 1) < 1e-6, axis=-1
+        )
         return symmetric & positive & unit_variance
 
     def feasible_like(self, prototype):
-        return jax.numpy.broadcast_to(jax.numpy.eye(prototype.shape[-1]), prototype.shape)
+        return jax.numpy.broadcast_to(
+            jax.numpy.eye(prototype.shape[-1]), prototype.shape
+        )
 
 
 class _Dependent(Constraint):
+    """
+    Placeholder for variables whose support depends on other variables.
+    These variables obey no simple coordinate-wise constraints.
+
+    :param bool is_discrete: Optional value of ``.is_discrete`` in case this
+        can be computed statically. If not provided, access to the
+        ``.is_discrete`` attribute will raise a NotImplementedError.
+    :param int event_dim: Optional value of ``.event_dim`` in case this can be
+        computed statically. If not provided, access to the ``.event_dim``
+        attribute will raise a NotImplementedError.
+    """
+
+    def __init__(self, *, is_discrete=NotImplemented, event_dim=NotImplemented):
+        self._is_discrete = is_discrete
+        self._event_dim = event_dim
+        super().__init__()
+
+    @property
+    def is_discrete(self):
+        if self._is_discrete is NotImplemented:
+            raise NotImplementedError(".is_discrete cannot be determined statically")
+        return self._is_discrete
+
+    @property
+    def event_dim(self):
+        if self._event_dim is NotImplemented:
+            raise NotImplementedError(".event_dim cannot be determined statically")
+        return self._event_dim
+
+    def __call__(self, x=None, *, is_discrete=NotImplemented, event_dim=NotImplemented):
+        if x is not None:
+            raise ValueError("Cannot determine validity of dependent constraint")
+
+        # Support for syntax to customize static attributes::
+        #     constraints.dependent(is_discrete=True, event_dim=1)
+        if is_discrete is NotImplemented:
+            is_discrete = self._is_discrete
+        if event_dim is NotImplemented:
+            event_dim = self._event_dim
+        return _Dependent(is_discrete=is_discrete, event_dim=event_dim)
+
+
+class dependent_property(property, _Dependent):
+    def __init__(
+        self, fn=None, *, is_discrete=NotImplemented, event_dim=NotImplemented
+    ):
+        super().__init__(fn)
+        self._is_discrete = is_discrete
+        self._event_dim = event_dim
+
     def __call__(self, x):
-        raise ValueError('Cannot determine validity of dependent constraint')
+        if not callable(x):
+            return super().__call__(x)
+
+        # Support for syntax to customize static attributes::
+        #     @constraints.dependent_property(is_discrete=True, event_dim=1)
+        #     def support(self):
+        #         ...
+        return dependent_property(
+            x, is_discrete=self._is_discrete, event_dim=self._event_dim
+        )
 
 
 def is_dependent(constraint):
@@ -148,12 +222,15 @@ class _IndependentConstraint(Constraint):
     dims in :meth:`check`, so that an event is valid only if all its
     independent entries are valid.
     """
+
     def __init__(self, base_constraint, reinterpreted_batch_ndims):
         assert isinstance(base_constraint, Constraint)
         assert isinstance(reinterpreted_batch_ndims, int)
         assert reinterpreted_batch_ndims >= 0
         if isinstance(base_constraint, _IndependentConstraint):
-            reinterpreted_batch_ndims = reinterpreted_batch_ndims + base_constraint.reinterpreted_batch_ndims
+            reinterpreted_batch_ndims = (
+                reinterpreted_batch_ndims + base_constraint.reinterpreted_batch_ndims
+            )
             base_constraint = base_constraint.base_constraint
         self.base_constraint = base_constraint
         self.reinterpreted_batch_ndims = reinterpreted_batch_ndims
@@ -169,9 +246,15 @@ class _IndependentConstraint(Constraint):
             return result
         elif jax.numpy.ndim(result) < self.reinterpreted_batch_ndims:
             expected = self.event_dim
-            raise ValueError(f"Expected value.dim() >= {expected} but got {jax.numpy.ndim(value)}")
+            raise ValueError(
+                f"Expected value.dim() >= {expected} but got {jax.numpy.ndim(value)}"
+            )
         result = result.reshape(
-            jax.numpy.shape(result)[:jax.numpy.ndim(result) - self.reinterpreted_batch_ndims] + (-1,))
+            jax.numpy.shape(result)[
+                : jax.numpy.ndim(result) - self.reinterpreted_batch_ndims
+            ]
+            + (-1,)
+        )
         result = result.all(-1)
         return result
 
@@ -191,6 +274,8 @@ class _LessThan(Constraint):
 
 
 class _IntegerInterval(Constraint):
+    is_discrete = True
+
     def __init__(self, lower_bound, upper_bound):
         self.lower_bound = lower_bound
         self.upper_bound = upper_bound
@@ -203,6 +288,8 @@ class _IntegerInterval(Constraint):
 
 
 class _IntegerGreaterThan(Constraint):
+    is_discrete = True
+
     def __init__(self, lower_bound):
         self.lower_bound = lower_bound
 
@@ -222,7 +309,9 @@ class _Interval(Constraint):
         return (x >= self.lower_bound) & (x <= self.upper_bound)
 
     def feasible_like(self, prototype):
-        return jax.numpy.broadcast_to((self.lower_bound + self.upper_bound) / 2, jax.numpy.shape(prototype))
+        return jax.numpy.broadcast_to(
+            (self.lower_bound + self.upper_bound) / 2, jax.numpy.shape(prototype)
+        )
 
 
 class _LowerCholesky(Constraint):
@@ -231,15 +320,20 @@ class _LowerCholesky(Constraint):
     def __call__(self, x):
         jnp = np if isinstance(x, (np.ndarray, np.generic)) else jax.numpy
         tril = jnp.tril(x)
-        lower_triangular = jnp.all(jnp.reshape(tril == x, x.shape[:-2] + (-1,)), axis=-1)
+        lower_triangular = jnp.all(
+            jnp.reshape(tril == x, x.shape[:-2] + (-1,)), axis=-1
+        )
         positive_diagonal = jnp.all(jnp.diagonal(x, axis1=-2, axis2=-1) > 0, axis=-1)
         return lower_triangular & positive_diagonal
 
     def feasible_like(self, prototype):
-        return jax.numpy.broadcast_to(jax.numpy.eye(prototype.shape[-1]), prototype.shape)
+        return jax.numpy.broadcast_to(
+            jax.numpy.eye(prototype.shape[-1]), prototype.shape
+        )
 
 
 class _Multinomial(Constraint):
+    is_discrete = True
     event_dim = 1
 
     def __init__(self, upper_bound):
@@ -249,7 +343,9 @@ class _Multinomial(Constraint):
         return (x >= 0).all(axis=-1) & (x.sum(axis=-1) == self.upper_bound)
 
     def feasible_like(self, prototype):
-        pad_width = ((0, 0),) * jax.numpy.ndim(self.upper_bound) + ((0, prototype.shape[-1] - 1),)
+        pad_width = ((0, 0),) * jax.numpy.ndim(self.upper_bound) + (
+            (0, prototype.shape[-1] - 1),
+        )
         value = jax.numpy.pad(jax.numpy.expand_dims(self.upper_bound, -1), pad_width)
         return jax.numpy.broadcast_to(value, prototype.shape)
 
@@ -261,7 +357,9 @@ class _OrderedVector(Constraint):
         return (x[..., 1:] > x[..., :-1]).all(axis=-1)
 
     def feasible_like(self, prototype):
-        return jax.numpy.broadcast_to(jax.numpy.arange(float(prototype.shape[-1])), prototype.shape)
+        return jax.numpy.broadcast_to(
+            jax.numpy.arange(float(prototype.shape[-1])), prototype.shape
+        )
 
 
 class _PositiveDefinite(Constraint):
@@ -276,13 +374,32 @@ class _PositiveDefinite(Constraint):
         return symmetric & positive
 
     def feasible_like(self, prototype):
-        return jax.numpy.broadcast_to(jax.numpy.eye(prototype.shape[-1]), prototype.shape)
+        return jax.numpy.broadcast_to(
+            jax.numpy.eye(prototype.shape[-1]), prototype.shape
+        )
+
+
+class _PositiveOrderedVector(Constraint):
+    """
+    Constrains to a positive real-valued tensor where the elements are monotonically
+    increasing along the `event_shape` dimension.
+    """
+
+    event_dim = 1
+
+    def __call__(self, x):
+        return ordered_vector.check(x) & independent(positive, 1).check(x)
+
+    def feasible_like(self, prototype):
+        return jax.numpy.broadcast_to(
+            jax.numpy.exp(jax.numpy.arange(float(prototype.shape[-1]))), prototype.shape
+        )
 
 
 class _Real(Constraint):
     def __call__(self, x):
         # XXX: consider to relax this condition to [-inf, inf] interval
-        return (x == x) & (x != float('inf')) & (x != float('-inf'))
+        return (x == x) & (x != float("inf")) & (x != float("-inf"))
 
     def feasible_like(self, prototype):
         return jax.numpy.zeros_like(prototype)
@@ -299,7 +416,42 @@ class _Simplex(Constraint):
         return jax.numpy.full_like(prototype, 1 / prototype.shape[-1])
 
 
+class _SoftplusPositive(_GreaterThan):
+    def __init__(self):
+        super().__init__(lower_bound=0.0)
+
+    def feasible_like(self, prototype):
+        return jax.numpy.full(jax.numpy.shape(prototype), np.log(2))
+
+
+class _SoftplusLowerCholesky(_LowerCholesky):
+    def feasible_like(self, prototype):
+        return jax.numpy.broadcast_to(
+            jax.numpy.eye(prototype.shape[-1]) * np.log(2), prototype.shape
+        )
+
+
+class _Sphere(Constraint):
+    """
+    Constrain to the Euclidean sphere of any dimension.
+    """
+
+    event_dim = 1
+    reltol = 10.0  # Relative to finfo.eps.
+
+    def __call__(self, x):
+        jnp = np if isinstance(x, (np.ndarray, np.generic)) else jax.numpy
+        eps = jnp.finfo(x.dtype).eps
+        norm = jnp.linalg.norm(x, axis=-1)
+        error = jnp.abs(norm - 1)
+        return error < self.reltol * eps * x.shape[-1] ** 0.5
+
+    def feasible_like(self, prototype):
+        return jax.numpy.full_like(prototype, prototype.shape[-1] ** (-0.5))
+
+
 # TODO: Make types consistent
+# See https://github.com/pytorch/pytorch/issues/50616
 
 boolean = _Boolean()
 corr_cholesky = _CorrCholesky()
@@ -315,10 +467,14 @@ lower_cholesky = _LowerCholesky()
 multinomial = _Multinomial
 nonnegative_integer = _IntegerGreaterThan(0)
 ordered_vector = _OrderedVector()
-positive = _GreaterThan(0.)
+positive = _GreaterThan(0.0)
 positive_definite = _PositiveDefinite()
 positive_integer = _IntegerGreaterThan(1)
+positive_ordered_vector = _PositiveOrderedVector()
 real = _Real()
 real_vector = independent(real, 1)
 simplex = _Simplex()
-unit_interval = _Interval(0., 1.)
+softplus_lower_cholesky = _SoftplusLowerCholesky()
+softplus_positive = _SoftplusPositive()
+sphere = _Sphere()
+unit_interval = _Interval(0.0, 1.0)
