@@ -7,10 +7,9 @@ from operator import attrgetter
 import os
 import warnings
 
-from jax import device_put, jit, lax, pmap, random, vmap
+from jax import jit, lax, local_device_count, pmap, random, vmap
 from jax.core import Tracer
 from jax.interpreters.xla import DeviceArray
-from jax.lib import xla_bridge
 import jax.numpy as jnp
 from jax.tree_util import tree_flatten, tree_map, tree_multimap
 
@@ -18,8 +17,8 @@ from numpyro.diagnostics import print_summary
 from numpyro.util import cached_by, fori_collect, identity
 
 __all__ = [
-    'MCMCKernel',
-    'MCMC',
+    "MCMCKernel",
+    "MCMC",
 ]
 
 
@@ -68,6 +67,7 @@ class MCMCKernel(ABC):
         >>> samples = mcmc.get_samples()
         >>> mcmc.print_summary()  # doctest: +SKIP
     """
+
     def postprocess_fn(self, model_args, model_kwargs):
         """
         Get a function that transforms unconstrained values at sample sites to values
@@ -138,13 +138,13 @@ class MCMCKernel(ABC):
         Given the current `state`, returns the diagnostics string to
         be added to progress bar for diagnostics purpose.
         """
-        return ''
+        return ""
 
 
 def _get_progbar_desc_str(num_warmup, phase, i):
     if phase is not None:
         return phase
-    return 'warmup' if i < num_warmup else 'sample'
+    return "warmup" if i < num_warmup else "sample"
 
 
 def _get_value_from_index(xs, i):
@@ -169,7 +169,7 @@ def _sample_fn_jit_args(state, sampler):
 
 def _sample_fn_nojit_args(state, sampler, args, kwargs):
     # state is a tuple of size 1 - containing HMCState
-    return sampler.sample(state[0], args, kwargs),
+    return (sampler.sample(state[0], args, kwargs),)
 
 
 def _collect_fn(collect_fields):
@@ -201,7 +201,13 @@ class MCMC(object):
 
     .. note:: `chain_method` is an experimental arg, which might be removed in a future version.
 
-    .. note:: Setting `progress_bar=False` will improve the speed for many cases.
+    .. note:: Setting `progress_bar=False` will improve the speed for many cases. But it might
+        require more memory than the other option.
+
+    .. note:: If setting `num_chains` greater than `1` in a Jupyter Notebook, then you will need to
+        have installed `ipywidgets <https://ipywidgets.readthedocs.io/en/latest/user_install.html>`_
+        in the environment from which you launced Jupyter in order for the progress bars to render
+        correctly.
 
     :param MCMCKernel sampler: an instance of :class:`~numpyro.infer.mcmc.MCMCKernel` that
         determines the sampler for running MCMC. Currently, only :class:`~numpyro.infer.hmc.HMC`
@@ -211,9 +217,9 @@ class MCMC(object):
     :param int thinning: Positive integer that controls the fraction of post-warmup samples that are
         retained. For example if thinning is 2 then every other sample is retained.
         Defaults to 1, i.e. no thinning.
-    :param int num_chains: Number of Number of MCMC chains to run. By default,
-        chains will be run in parallel using :func:`jax.pmap`, failing which,
-        chains will be run in sequence.
+    :param int num_chains: Number of MCMC chains to run. By default, chains will be
+        run in parallel using :func:`jax.pmap`. If there are not enough devices
+        available, chains will be run in sequence.
     :param postprocess_fn: Post-processing callable - used to convert a collection of unconstrained
         sample values returned from the sampler to constrained values that lie within the support
         of the sample sites. Additionally, this is used to return values at deterministic sites in
@@ -229,16 +235,20 @@ class MCMC(object):
         computation as a function of model arguments. As such, calling `MCMC.run` again
         on a same sized but different dataset will not result in additional compilation cost.
     """
-    def __init__(self,
-                 sampler,
-                 num_warmup,
-                 num_samples,
-                 num_chains=1,
-                 thinning=1,
-                 postprocess_fn=None,
-                 chain_method='parallel',
-                 progress_bar=True,
-                 jit_model_args=False):
+
+    def __init__(
+        self,
+        sampler,
+        *,
+        num_warmup,
+        num_samples,
+        num_chains=1,
+        thinning=1,
+        postprocess_fn=None,
+        chain_method="parallel",
+        progress_bar=True,
+        jit_model_args=False,
+    ):
         self.sampler = sampler
         self._sample_field = sampler.sample_field
         self._default_fields = sampler.default_fields
@@ -246,17 +256,28 @@ class MCMC(object):
         self.num_samples = num_samples
         self.num_chains = num_chains
         if not isinstance(thinning, int) or thinning < 1:
-            raise ValueError('thinning must be a positive integer')
+            raise ValueError("thinning must be a positive integer")
         self.thinning = thinning
         self.postprocess_fn = postprocess_fn
-        if chain_method not in ['parallel', 'vectorized', 'sequential']:
-            raise ValueError('Only supporting the following methods to draw chains:'
-                             ' "sequential", "parallel", or "vectorized"')
+        if chain_method not in ["parallel", "vectorized", "sequential"]:
+            raise ValueError(
+                "Only supporting the following methods to draw chains:"
+                ' "sequential", "parallel", or "vectorized"'
+            )
+        if chain_method == "parallel" and local_device_count() < self.num_chains:
+            chain_method = "sequential"
+            warnings.warn(
+                "There are not enough devices to run parallel chains: expected {} but got {}."
+                " Chains will be drawn sequentially. If you are running MCMC in CPU,"
+                " consider using `numpyro.set_host_device_count({})` at the beginning"
+                " of your program. You can double-check how many devices are available in"
+                " your system using `jax.local_device_count()`.".format(
+                    self.num_chains, local_device_count(), self.num_chains
+                )
+            )
         self.chain_method = chain_method
         self.progress_bar = progress_bar
-        # TODO: We should have progress bars (maybe without diagnostics) for num_chains > 1
-        if (chain_method == 'parallel' and num_chains > 1) or (
-                "CI" in os.environ or "PYTEST_XDIST_WORKER" in os.environ):
+        if "CI" in os.environ or "PYTEST_XDIST_WORKER" in os.environ:
             self.progress_bar = False
         self._jit_model_args = jit_model_args
         self._states = None
@@ -276,7 +297,9 @@ class MCMC(object):
             args, kwargs = (None,), (None,)
         else:
             args = tree_map(lambda x: _hashable(x), self._args)
-            kwargs = tree_map(lambda x: _hashable(x), tuple(sorted(self._kwargs.items())))
+            kwargs = tree_map(
+                lambda x: _hashable(x), tuple(sorted(self._kwargs.items()))
+            )
         key = args + kwargs
         try:
             fns = self._cache.get(key, None)
@@ -300,10 +323,15 @@ class MCMC(object):
                 sample_fn = partial(_sample_fn_jit_args, sampler=self.sampler)
                 postprocess_fn = jit(laxmap_postprocess_fn)
             else:
-                sample_fn = partial(_sample_fn_nojit_args, sampler=self.sampler,
-                                    args=self._args, kwargs=self._kwargs)
-                postprocess_fn = jit(partial(laxmap_postprocess_fn,
-                                             args=self._args, kwargs=self._kwargs))
+                sample_fn = partial(
+                    _sample_fn_nojit_args,
+                    sampler=self.sampler,
+                    args=self._args,
+                    kwargs=self._kwargs,
+                )
+                postprocess_fn = jit(
+                    partial(laxmap_postprocess_fn, args=self._args, kwargs=self._kwargs)
+                )
 
             fns = sample_fn, postprocess_fn
             if key is not None:
@@ -324,27 +352,43 @@ class MCMC(object):
     def _single_chain_mcmc(self, init, args, kwargs, collect_fields):
         rng_key, init_state, init_params = init
         if init_state is None:
-            init_state = self.sampler.init(rng_key, self.num_warmup, init_params,
-                                           model_args=args, model_kwargs=kwargs)
+            init_state = self.sampler.init(
+                rng_key,
+                self.num_warmup,
+                init_params,
+                model_args=args,
+                model_kwargs=kwargs,
+            )
         sample_fn, postprocess_fn = self._get_cached_fns()
-        diagnostics = lambda x: self.sampler.get_diagnostics_str(x[0]) if rng_key.ndim == 1 else ''   # noqa: E731
+        diagnostics = (
+            lambda x: self.sampler.get_diagnostics_str(x[0])
+            if rng_key.ndim == 1
+            else ""
+        )  # noqa: E731
         init_val = (init_state, args, kwargs) if self._jit_model_args else (init_state,)
         lower_idx = self._collection_params["lower"]
         upper_idx = self._collection_params["upper"]
         phase = self._collection_params["phase"]
         collection_size = self._collection_params["collection_size"]
-        collection_size = collection_size if collection_size is None else collection_size // self.thinning
-        collect_vals = fori_collect(lower_idx,
-                                    upper_idx,
-                                    sample_fn,
-                                    init_val,
-                                    transform=_collect_fn(collect_fields),
-                                    progbar=self.progress_bar,
-                                    return_last_val=True,
-                                    thinning=self.thinning,
-                                    collection_size=collection_size,
-                                    progbar_desc=partial(_get_progbar_desc_str, lower_idx, phase),
-                                    diagnostics_fn=diagnostics)
+        collection_size = (
+            collection_size
+            if collection_size is None
+            else collection_size // self.thinning
+        )
+        collect_vals = fori_collect(
+            lower_idx,
+            upper_idx,
+            sample_fn,
+            init_val,
+            transform=_collect_fn(collect_fields),
+            progbar=self.progress_bar,
+            return_last_val=True,
+            thinning=self.thinning,
+            collection_size=collection_size,
+            progbar_desc=partial(_get_progbar_desc_str, lower_idx, phase),
+            diagnostics_fn=diagnostics,
+            num_chains=self.num_chains if self.chain_method == "parallel" else 1,
+        )
         states, last_val = collect_vals
         # Get first argument of type `HMCState`
         last_state = last_val[0]
@@ -358,20 +402,28 @@ class MCMC(object):
         # (which happens when lower_idx==upper_idx)
         if len(site_values) > 0 and jnp.shape(site_values[0])[0] > 0:
             if self._jit_model_args:
-                states[self._sample_field] = postprocess_fn(states[self._sample_field], args, kwargs)
+                states[self._sample_field] = postprocess_fn(
+                    states[self._sample_field], args, kwargs
+                )
             else:
                 states[self._sample_field] = postprocess_fn(states[self._sample_field])
         return states, last_state
 
-    def _set_collection_params(self, lower=None, upper=None, collection_size=None, phase=None):
+    def _set_collection_params(
+        self, lower=None, upper=None, collection_size=None, phase=None
+    ):
         self._collection_params["lower"] = self.num_warmup if lower is None else lower
-        self._collection_params["upper"] = self.num_warmup + self.num_samples if upper is None else upper
+        self._collection_params["upper"] = (
+            self.num_warmup + self.num_samples if upper is None else upper
+        )
         self._collection_params["collection_size"] = collection_size
         self._collection_params["phase"] = phase
 
     def _compile(self, rng_key, *args, extra_fields=(), init_params=None, **kwargs):
         self._set_collection_params(0, 0, self.num_samples)
-        self.run(rng_key, *args, extra_fields=extra_fields, init_params=init_params, **kwargs)
+        self.run(
+            rng_key, *args, extra_fields=extra_fields, init_params=init_params, **kwargs
+        )
         rng_key = (_hashable(rng_key),)
         args = tree_map(lambda x: _hashable(x), args)
         kwargs = tree_map(lambda x: _hashable(x), tuple(sorted(kwargs.items())))
@@ -393,7 +445,7 @@ class MCMC(object):
 
             .. code-block:: python
 
-                mcmc = MCMC(NUTS(model), 100, 100)
+                mcmc = MCMC(NUTS(model), num_warmup=100, num_samples=100)
                 mcmc.run(random.PRNGKey(0))
                 first_100_samples = mcmc.get_samples()
                 mcmc.post_warmup_state = mcmc.last_state
@@ -413,7 +465,15 @@ class MCMC(object):
         """
         return self._last_state
 
-    def warmup(self, rng_key, *args, extra_fields=(), collect_warmup=False, init_params=None, **kwargs):
+    def warmup(
+        self,
+        rng_key,
+        *args,
+        extra_fields=(),
+        collect_warmup=False,
+        init_params=None,
+        **kwargs,
+    ):
         """
         Run the MCMC warmup adaptation phase. After this call, `self.warmup_state` will be set
         and the :meth:`run` method will skip the warmup adaptation phase. To run `warmup` again
@@ -437,8 +497,12 @@ class MCMC(object):
         if collect_warmup:
             self._set_collection_params(0, self.num_warmup, self.num_warmup, "warmup")
         else:
-            self._set_collection_params(self.num_warmup, self.num_warmup, self.num_samples, "warmup")
-        self.run(rng_key, *args, extra_fields=extra_fields, init_params=init_params, **kwargs)
+            self._set_collection_params(
+                self.num_warmup, self.num_warmup, self.num_samples, "warmup"
+            )
+        self.run(
+            rng_key, *args, extra_fields=extra_fields, init_params=init_params, **kwargs
+        )
         self._warmup_state = self._last_state
 
     def run(self, rng_key, *args, extra_fields=(), init_params=None, **kwargs):
@@ -463,6 +527,9 @@ class MCMC(object):
             See https://jax.readthedocs.io/en/latest/async_dispatch.html and
             https://jax.readthedocs.io/en/latest/profiling.html for pointers on profiling jax programs.
         """
+        init_params = tree_map(
+            lambda x: lax.convert_element_type(x, jnp.result_type(x)), init_params
+        )
         self._args = args
         self._kwargs = kwargs
         init_state = self._get_cached_init_state(rng_key, args, kwargs)
@@ -473,47 +540,44 @@ class MCMC(object):
             self._set_collection_params(0, self.num_samples, self.num_samples, "sample")
             init_state = self._warmup_state._replace(rng_key=rng_key)
 
-        chain_method = self.chain_method
-        if chain_method == 'parallel' and xla_bridge.device_count() < self.num_chains:
-            chain_method = 'sequential'
-            warnings.warn('There are not enough devices to run parallel chains: expected {} but got {}.'
-                          ' Chains will be drawn sequentially. If you are running MCMC in CPU,'
-                          ' consider to use `numpyro.set_host_device_count({})` at the beginning'
-                          ' of your program.'
-                          .format(self.num_chains, xla_bridge.device_count(), self.num_chains))
-
         if init_params is not None and self.num_chains > 1:
             prototype_init_val = tree_flatten(init_params)[0][0]
             if jnp.shape(prototype_init_val)[0] != self.num_chains:
-                raise ValueError('`init_params` must have the same leading dimension'
-                                 ' as `num_chains`.')
+                raise ValueError(
+                    "`init_params` must have the same leading dimension"
+                    " as `num_chains`."
+                )
         assert isinstance(extra_fields, (tuple, list))
-        collect_fields = tuple(set((self._sample_field,) + tuple(self._default_fields) +
-                                   tuple(extra_fields)))
-        partial_map_fn = partial(self._single_chain_mcmc,
-                                 args=args,
-                                 kwargs=kwargs,
-                                 collect_fields=collect_fields)
+        collect_fields = tuple(
+            set(
+                (self._sample_field,)
+                + tuple(self._default_fields)
+                + tuple(extra_fields)
+            )
+        )
+        partial_map_fn = partial(
+            self._single_chain_mcmc,
+            args=args,
+            kwargs=kwargs,
+            collect_fields=collect_fields,
+        )
         map_args = (rng_key, init_state, init_params)
         if self.num_chains == 1:
             states_flat, last_state = partial_map_fn(map_args)
             states = tree_map(lambda x: x[jnp.newaxis, ...], states_flat)
         else:
-            if chain_method == 'sequential':
-                if self.progress_bar:
-                    states, last_state = _laxmap(partial_map_fn, map_args)
-                else:
-                    states, last_state = lax.map(partial_map_fn, map_args)
-            elif chain_method == 'parallel':
+            if self.chain_method == "sequential":
+                states, last_state = _laxmap(partial_map_fn, map_args)
+            elif self.chain_method == "parallel":
                 states, last_state = pmap(partial_map_fn)(map_args)
-                # TODO: remove when https://github.com/google/jax/issues/3597 is resolved
-                states = device_put(states)
             else:
-                assert chain_method == 'vectorized'
+                assert self.chain_method == "vectorized"
                 states, last_state = partial_map_fn(map_args)
                 # swap num_samples x num_chains to num_chains x num_samples
                 states = tree_map(lambda x: jnp.swapaxes(x, 0, 1), states)
-            states_flat = tree_map(lambda x: jnp.reshape(x, (-1,) + x.shape[2:]), states)
+            states_flat = tree_map(
+                lambda x: jnp.reshape(x, (-1,) + x.shape[2:]), states
+            )
         self._last_state = last_state
         self._states = states
         self._states_flat = states_flat
@@ -530,8 +594,11 @@ class MCMC(object):
             but can be any :func:`jaxlib.pytree`, more generally (e.g. when defining a
             `potential_fn` for HMC that takes `list` args).
         """
-        return self._states[self._sample_field] if group_by_chain \
+        return (
+            self._states[self._sample_field]
+            if group_by_chain
             else self._states_flat[self._sample_field]
+        )
 
     def get_extra_fields(self, group_by_chain=False):
         """
@@ -563,9 +630,19 @@ class MCMC(object):
             # they can have different key names, not necessary due to deterministic
             # behavior. We might revise this logic if needed in the future.
             if isinstance(state_sample_field, dict):
-                sites = {k: v for k, v in self._states[self._sample_field].items()
-                         if k in state_sample_field}
+                sites = {
+                    k: v
+                    for k, v in self._states[self._sample_field].items()
+                    if k in state_sample_field
+                }
         print_summary(sites, prob=prob)
         extra_fields = self.get_extra_fields()
-        if 'diverging' in extra_fields:
-            print("Number of divergences: {}".format(jnp.sum(extra_fields['diverging'])))
+        if "diverging" in extra_fields:
+            print(
+                "Number of divergences: {}".format(jnp.sum(extra_fields["diverging"]))
+            )
+
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        state["_cache"] = {}
+        return state
