@@ -24,7 +24,9 @@ def _get_proposal_loc_and_scale(samples, loc, scale, new_sample):
     if scale.ndim > loc.ndim:
         new_scale = cholesky_update(scale, new_sample - loc, weight)
         proposal_scale = cholesky_update(new_scale, samples - loc, -weight)
-        proposal_scale = cholesky_update(proposal_scale, new_sample - samples, - (weight ** 2))
+        proposal_scale = cholesky_update(
+            proposal_scale, new_sample - samples, -(weight ** 2)
+        )
     else:
         var = jnp.square(scale) + weight * jnp.square(new_sample - loc)
         proposal_var = var - weight * jnp.square(samples - loc)
@@ -46,9 +48,20 @@ def _sample_proposal(inv_mass_matrix_sqrt, rng_key, batch_shape=()):
     return r
 
 
-SAAdaptState = namedtuple('SAAdaptState', ['zs', 'pes', 'loc', 'inv_mass_matrix_sqrt'])
-SAState = namedtuple('SAState', ['i', 'z', 'potential_energy', 'accept_prob',
-                                 'mean_accept_prob', 'diverging', 'adapt_state', 'rng_key'])
+SAAdaptState = namedtuple("SAAdaptState", ["zs", "pes", "loc", "inv_mass_matrix_sqrt"])
+SAState = namedtuple(
+    "SAState",
+    [
+        "i",
+        "z",
+        "potential_energy",
+        "accept_prob",
+        "mean_accept_prob",
+        "diverging",
+        "adapt_state",
+        "rng_key",
+    ],
+)
 """
 A :func:`~collections.namedtuple` used in Sample Adaptive MCMC.
 This consists of the following fields:
@@ -87,22 +100,26 @@ def _numpy_delete(x, idx):
 # TODO: consider to expose this functional style
 def _sa(potential_fn=None, potential_fn_gen=None):
     wa_steps = None
-    max_delta_energy = 1000.
+    max_delta_energy = 1000.0
 
-    def init_kernel(init_params,
-                    num_warmup,
-                    adapt_state_size=None,
-                    inverse_mass_matrix=None,
-                    dense_mass=False,
-                    model_args=(),
-                    model_kwargs=None,
-                    rng_key=random.PRNGKey(0)):
+    def init_kernel(
+        init_params,
+        num_warmup,
+        adapt_state_size=None,
+        inverse_mass_matrix=None,
+        dense_mass=False,
+        model_args=(),
+        model_kwargs=None,
+        rng_key=random.PRNGKey(0),
+    ):
         nonlocal wa_steps
         wa_steps = num_warmup
         pe_fn = potential_fn
         if potential_fn_gen:
             if pe_fn is not None:
-                raise ValueError('Only one of `potential_fn` or `potential_fn_gen` must be provided.')
+                raise ValueError(
+                    "Only one of `potential_fn` or `potential_fn_gen` must be provided."
+                )
             else:
                 kwargs = {} if model_kwargs is None else model_kwargs
                 pe_fn = potential_fn_gen(*model_args, **kwargs)
@@ -110,16 +127,25 @@ def _sa(potential_fn=None, potential_fn_gen=None):
         z = init_params
         z_flat, unravel_fn = ravel_pytree(z)
         if inverse_mass_matrix is None:
-            inverse_mass_matrix = jnp.identity(z_flat.shape[-1]) if dense_mass else jnp.ones(z_flat.shape[-1])
-        inv_mass_matrix_sqrt = jnp.linalg.cholesky(inverse_mass_matrix) if dense_mass \
+            inverse_mass_matrix = (
+                jnp.identity(z_flat.shape[-1])
+                if dense_mass
+                else jnp.ones(z_flat.shape[-1])
+            )
+        inv_mass_matrix_sqrt = (
+            jnp.linalg.cholesky(inverse_mass_matrix)
+            if dense_mass
             else jnp.sqrt(inverse_mass_matrix)
+        )
         if adapt_state_size is None:
             # XXX: heuristic choice
             adapt_state_size = 2 * z_flat.shape[-1]
         else:
-            assert adapt_state_size > 1, 'adapt_state_size should be greater than 1.'
+            assert adapt_state_size > 1, "adapt_state_size should be greater than 1."
         # NB: mean is init_params
-        zs = z_flat + _sample_proposal(inv_mass_matrix_sqrt, rng_key_zs, (adapt_state_size,))
+        zs = z_flat + _sample_proposal(
+            inv_mass_matrix_sqrt, rng_key_zs, (adapt_state_size,)
+        )
         # compute potential energies
         pes = lax.map(lambda z: pe_fn(unravel_fn(z)), zs)
         if dense_mass:
@@ -128,14 +154,25 @@ def _sa(potential_fn=None, potential_fn_gen=None):
                 cov = cov.reshape((1, 1))
             cholesky = jnp.linalg.cholesky(cov)
             # if cholesky is NaN, we use the scale from `sample_proposal` here
-            inv_mass_matrix_sqrt = jnp.where(jnp.any(jnp.isnan(cholesky)), inv_mass_matrix_sqrt, cholesky)
+            inv_mass_matrix_sqrt = jnp.where(
+                jnp.any(jnp.isnan(cholesky)), inv_mass_matrix_sqrt, cholesky
+            )
         else:
             inv_mass_matrix_sqrt = jnp.std(zs, 0)
         adapt_state = SAAdaptState(zs, pes, jnp.mean(zs, 0), inv_mass_matrix_sqrt)
         k = random.categorical(rng_key_z, jnp.zeros(zs.shape[0]))
         z = unravel_fn(zs[k])
         pe = pes[k]
-        sa_state = SAState(0, z, pe, 0., 0., False, adapt_state, rng_key_sa)
+        sa_state = SAState(
+            jnp.array(0),
+            z,
+            pe,
+            jnp.zeros(()),
+            jnp.zeros(()),
+            jnp.array(False),
+            adapt_state,
+            rng_key_sa,
+        )
         return device_put(sa_state)
 
     def sample_kernel(sa_state, model_args=(), model_kwargs=None):
@@ -156,7 +193,9 @@ def _sa(potential_fn=None, potential_fn_gen=None):
         else:
             scale = jnp.std(zs, 0)
 
-        rng_key, rng_key_z, rng_key_reject, rng_key_accept = random.split(sa_state.rng_key, 4)
+        rng_key, rng_key_z, rng_key_reject, rng_key_accept = random.split(
+            sa_state.rng_key, 4
+        )
         _, unravel_fn = ravel_pytree(sa_state.z)
 
         z = loc + _sample_proposal(scale, rng_key_z)
@@ -172,7 +211,9 @@ def _sa(potential_fn=None, potential_fn_gen=None):
         locs_ = jnp.concatenate([locs, loc[None, :]])
         scales_ = jnp.concatenate([scales, scale[None, ...]])
         if scale.ndim == 2:  # dense_mass
-            log_weights_ = dist.MultivariateNormal(locs_, scale_tril=scales_).log_prob(zs_) + pes_
+            log_weights_ = (
+                dist.MultivariateNormal(locs_, scale_tril=scales_).log_prob(zs_) + pes_
+            )
         else:
             log_weights_ = dist.Normal(locs_, scales_).log_prob(zs_).sum(-1) + pes_
         # mask invalid values (nan, +inf) by -inf
@@ -189,7 +230,9 @@ def _sa(potential_fn=None, potential_fn_gen=None):
         accept_prob = 1 - jnp.exp(log_weights_[-1] - logsumexp(log_weights_))
         itr = sa_state.i + 1
         n = jnp.where(sa_state.i < wa_steps, itr, itr - wa_steps)
-        mean_accept_prob = sa_state.mean_accept_prob + (accept_prob - sa_state.mean_accept_prob) / n
+        mean_accept_prob = (
+            sa_state.mean_accept_prob + (accept_prob - sa_state.mean_accept_prob) / n
+        )
 
         # XXX: we make a modification of SA sampler in [1]
         # in [1], each MCMC state contains N points `zs`
@@ -197,7 +240,9 @@ def _sa(potential_fn=None, potential_fn_gen=None):
         k = random.categorical(rng_key_accept, jnp.zeros(zs.shape[0]))
         z = unravel_fn(zs[k])
         pe = pes[k]
-        return SAState(itr, z, pe, accept_prob, mean_accept_prob, diverging, adapt_state, rng_key)
+        return SAState(
+            itr, z, pe, accept_prob, mean_accept_prob, diverging, adapt_state, rng_key
+        )
 
     return init_kernel, sample_kernel
 
@@ -236,10 +281,17 @@ class SA(MCMCKernel):
     :param callable init_strategy: a per-site initialization function.
         See :ref:`init_strategy` section for available functions.
     """
-    def __init__(self, model=None, potential_fn=None, adapt_state_size=None,
-                 dense_mass=True, init_strategy=init_to_uniform):
+
+    def __init__(
+        self,
+        model=None,
+        potential_fn=None,
+        adapt_state_size=None,
+        dense_mass=True,
+        init_strategy=init_to_uniform,
+    ):
         if not (model is None) ^ (potential_fn is None):
-            raise ValueError('Only one of `model` or `potential_fn` must be specified.')
+            raise ValueError("Only one of `model` or `potential_fn` must be specified.")
         self._model = model
         self._potential_fn = potential_fn
         self._adapt_state_size = adapt_state_size
@@ -258,7 +310,9 @@ class SA(MCMCKernel):
                 dynamic_args=True,
                 init_strategy=self._init_strategy,
                 model_args=model_args,
-                model_kwargs=model_kwargs)
+                model_kwargs=model_kwargs,
+                validate_grad=False,
+            )
             init_params = init_params[0]
             # NB: init args is different from HMC
             self._init_fn, sample_fn = _sa(potential_fn_gen=potential_fn)
@@ -272,19 +326,26 @@ class SA(MCMCKernel):
             self._sample_fn = sample_fn
         return init_params
 
-    def init(self, rng_key, num_warmup, init_params=None, model_args=(), model_kwargs={}):
+    def init(
+        self, rng_key, num_warmup, init_params=None, model_args=(), model_kwargs={}
+    ):
         # non-vectorized
         if rng_key.ndim == 1:
             rng_key, rng_key_init_model = random.split(rng_key)
         # vectorized
         else:
-            rng_key, rng_key_init_model = jnp.swapaxes(vmap(random.split)(rng_key), 0, 1)
+            rng_key, rng_key_init_model = jnp.swapaxes(
+                vmap(random.split)(rng_key), 0, 1
+            )
             # we need only a single key for initializing PE / constraints fn
             rng_key_init_model = rng_key_init_model[0]
-        init_params = self._init_state(rng_key_init_model, model_args, model_kwargs, init_params)
+        init_params = self._init_state(
+            rng_key_init_model, model_args, model_kwargs, init_params
+        )
         if self._potential_fn and init_params is None:
-            raise ValueError('Valid value of `init_params` must be provided with'
-                             ' `potential_fn`.')
+            raise ValueError(
+                "Valid value of `init_params` must be provided with" " `potential_fn`."
+            )
 
         # NB: init args is different from HMC
         sa_init_fn = lambda init_params, rng_key: self._init_fn(  # noqa: E731
@@ -306,14 +367,14 @@ class SA(MCMCKernel):
 
     @property
     def sample_field(self):
-        return 'z'
+        return "z"
 
     @property
     def default_fields(self):
-        return ('z', 'diverging')
+        return ("z", "diverging")
 
     def get_diagnostics_str(self, state):
-        return 'acc. prob={:.2f}'.format(state.mean_accept_prob)
+        return "acc. prob={:.2f}".format(state.mean_accept_prob)
 
     def postprocess_fn(self, args, kwargs):
         if self._postprocess_fn is None:
@@ -331,3 +392,11 @@ class SA(MCMCKernel):
         :return: Next `state` after running SA.
         """
         return self._sample_fn(state, model_args, model_kwargs)
+
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        state["_sample_fn"] = None
+        state["_init_fn"] = None
+        state["_postprocess_fn"] = None
+        state["_potential_fn_gen"] = None
+        return state
