@@ -1,23 +1,27 @@
 # Copyright Contributors to the Pyro project.
 # SPDX-License-Identifier: Apache-2.0
 
-from collections import OrderedDict, namedtuple
+from collections import OrderedDict
 from contextlib import contextmanager
 import os
 import random
 import re
+import warnings
 
 import numpy as np
 import tqdm
+from tqdm.auto import tqdm as tqdm_auto
 
 import jax
 from jax import device_put, jit, lax, ops, vmap
 from jax.core import Tracer
-from jax.dtypes import canonicalize_dtype
+from jax.experimental import host_callback
+from jax.flatten_util import ravel_pytree
 import jax.numpy as jnp
-from jax.tree_util import tree_flatten, tree_map, tree_unflatten
+from jax.tree_util import tree_flatten, tree_map
 
 _DISABLE_CONTROL_FLOW_PRIM = False
+_CHAIN_RE = re.compile(r"(?<=_)\d+$")  # e.g. get '3' from 'TFRT_CPU_3'
 
 
 def set_rng_seed(rng_seed):
@@ -38,8 +42,8 @@ def enable_x64(use_x64=True):
         else 32 bits.
     """
     if not use_x64:
-        use_x64 = os.getenv('JAX_ENABLE_X64', 0)
-    jax.config.update('jax_enable_x64', use_x64)
+        use_x64 = os.getenv("JAX_ENABLE_X64", 0)
+    jax.config.update("jax_enable_x64", use_x64)
 
 
 def set_platform(platform=None):
@@ -50,8 +54,8 @@ def set_platform(platform=None):
     :param str platform: either 'cpu', 'gpu', or 'tpu'.
     """
     if platform is None:
-        platform = os.getenv('JAX_PLATFORM_NAME', 'cpu')
-    jax.config.update('jax_platform_name', platform)
+        platform = os.getenv("JAX_PLATFORM_NAME", "cpu")
+    jax.config.update("jax_platform_name", platform)
 
 
 def set_host_device_count(n):
@@ -73,10 +77,13 @@ def set_host_device_count(n):
 
     :param int n: number of CPU devices to use.
     """
-    xla_flags = os.getenv('XLA_FLAGS', '').lstrip('--')
-    xla_flags = re.sub(r'xla_force_host_platform_device_count=.+\s', '', xla_flags).split()
-    os.environ['XLA_FLAGS'] = ' '.join(['--xla_force_host_platform_device_count={}'.format(n)]
-                                       + xla_flags)
+    xla_flags = os.getenv("XLA_FLAGS", "")
+    xla_flags = re.sub(
+        r"--xla_force_host_platform_device_count=\S+", "", xla_flags
+    ).split()
+    os.environ["XLA_FLAGS"] = " ".join(
+        ["--xla_force_host_platform_device_count={}".format(n)] + xla_flags
+    )
 
 
 @contextmanager
@@ -146,7 +153,7 @@ def identity(x, *args, **kwargs):
 def cached_by(outer_fn, *keys):
     # Restrict cache size to prevent ref cycles.
     max_size = 8
-    outer_fn._cache = getattr(outer_fn, '_cache', OrderedDict())
+    outer_fn._cache = getattr(outer_fn, "_cache", OrderedDict())
 
     def _wrapped(fn):
         fn_cache = outer_fn._cache
@@ -164,9 +171,99 @@ def cached_by(outer_fn, *keys):
     return _wrapped
 
 
-def fori_collect(lower, upper, body_fun, init_val, transform=identity,
-                 progbar=True, return_last_val=False, collection_size=None,
-                 thinning=1, **progbar_opts):
+def progress_bar_factory(num_samples, num_chains):
+    """Factory that builds a progress bar decorator along
+    with the `set_tqdm_description` and `close_tqdm` functions
+    """
+
+    if num_samples > 20:
+        print_rate = int(num_samples / 20)
+    else:
+        print_rate = 1
+
+    remainder = num_samples % print_rate
+
+    tqdm_bars = {}
+    finished_chains = []
+    for chain in range(num_chains):
+        tqdm_bars[chain] = tqdm_auto(range(num_samples), position=chain)
+        tqdm_bars[chain].set_description("Compiling.. ", refresh=True)
+
+    def _update_tqdm(arg, transform, device):
+        chain_match = _CHAIN_RE.search(str(device))
+        assert chain_match
+        chain = int(chain_match.group())
+        tqdm_bars[chain].set_description(f"Running chain {chain}", refresh=False)
+        tqdm_bars[chain].update(arg)
+
+    def _close_tqdm(arg, transform, device):
+        chain_match = _CHAIN_RE.search(str(device))
+        assert chain_match
+        chain = int(chain_match.group())
+        tqdm_bars[chain].update(arg)
+        finished_chains.append(chain)
+        if len(finished_chains) == num_chains:
+            for chain in range(num_chains):
+                tqdm_bars[chain].close()
+
+    def _update_progress_bar(iter_num):
+        """Updates tqdm progress bar of a JAX loop only if the iteration number is a multiple of the print_rate
+        Usage: carry = progress_bar((iter_num, print_rate), carry)
+        """
+
+        _ = lax.cond(
+            iter_num == 1,
+            lambda _: host_callback.id_tap(
+                _update_tqdm, 0, result=iter_num, tap_with_device=True
+            ),
+            lambda _: iter_num,
+            operand=None,
+        )
+        _ = lax.cond(
+            iter_num % print_rate == 0,
+            lambda _: host_callback.id_tap(
+                _update_tqdm, print_rate, result=iter_num, tap_with_device=True
+            ),
+            lambda _: iter_num,
+            operand=None,
+        )
+        _ = lax.cond(
+            iter_num == num_samples,
+            lambda _: host_callback.id_tap(
+                _close_tqdm, remainder, result=iter_num, tap_with_device=True
+            ),
+            lambda _: iter_num,
+            operand=None,
+        )
+
+    def progress_bar_fori_loop(func):
+        """Decorator that adds a progress bar to `body_fun` used in `lax.fori_loop`.
+        Note that `body_fun` must be looping over a tuple who's first element is `np.arange(num_samples)`.
+        This means that `iter_num` is the current iteration number
+        """
+
+        def wrapper_progress_bar(i, vals):
+            result = func(i, vals)
+            _update_progress_bar(i + 1)
+            return result
+
+        return wrapper_progress_bar
+
+    return progress_bar_fori_loop
+
+
+def fori_collect(
+    lower,
+    upper,
+    body_fun,
+    init_val,
+    transform=identity,
+    progbar=True,
+    return_last_val=False,
+    collection_size=None,
+    thinning=1,
+    **progbar_opts,
+):
     """
     This looping construct works like :func:`~jax.lax.fori_loop` but with the additional
     effect of collecting values from the loop body. In addition, this allows for
@@ -202,29 +299,49 @@ def fori_collect(lower, upper, body_fun, init_val, transform=identity,
     """
     assert lower <= upper
     assert thinning >= 1
-    collection_size = (upper - lower) // thinning if collection_size is None else collection_size
+    collection_size = (
+        (upper - lower) // thinning if collection_size is None else collection_size
+    )
     assert collection_size >= (upper - lower) // thinning
     init_val_flat, unravel_fn = ravel_pytree(transform(init_val))
     start_idx = lower + (upper - lower) % thinning
+    num_chains = progbar_opts.pop("num_chains", 1)
+    # host_callback does not work yet with multi-GPU platforms
+    # See: https://github.com/google/jax/issues/6447
+    if num_chains > 1 and jax.default_backend() == "gpu":
+        warnings.warn(
+            "We will disable progress bar because it does not work yet on multi-GPUs platforms."
+        )
+        progbar = False
 
     @cached_by(fori_collect, body_fun, transform)
     def _body_fn(i, vals):
         val, collection, start_idx, thinning = vals
         val = body_fun(val)
         idx = (i - start_idx) // thinning
-        collection = cond(idx >= 0,
-                          collection,
-                          lambda x: ops.index_update(x, idx, ravel_pytree(transform(val))[0]),
-                          collection,
-                          identity)
+        collection = cond(
+            idx >= 0,
+            collection,
+            lambda x: ops.index_update(x, idx, ravel_pytree(transform(val))[0]),
+            collection,
+            identity,
+        )
         return val, collection, start_idx, thinning
 
     collection = jnp.zeros((collection_size,) + init_val_flat.shape)
     if not progbar:
-        last_val, collection, _, _ = fori_loop(0, upper, _body_fn, (init_val, collection, start_idx, thinning))
+        last_val, collection, _, _ = fori_loop(
+            0, upper, _body_fn, (init_val, collection, start_idx, thinning)
+        )
+    elif num_chains > 1:
+        progress_bar_fori_loop = progress_bar_factory(upper, num_chains)
+        _body_fn_pbar = progress_bar_fori_loop(_body_fn)
+        last_val, collection, _, _ = fori_loop(
+            0, upper, _body_fn_pbar, (init_val, collection, start_idx, thinning)
+        )
     else:
-        diagnostics_fn = progbar_opts.pop('diagnostics_fn', None)
-        progbar_desc = progbar_opts.pop('progbar_desc', lambda x: '')
+        diagnostics_fn = progbar_opts.pop("diagnostics_fn", None)
+        progbar_desc = progbar_opts.pop("progbar_desc", lambda x: "")
 
         vals = (init_val, collection, device_put(start_idx), device_put(thinning))
         if upper == 0:
@@ -244,33 +361,6 @@ def fori_collect(lower, upper, body_fun, init_val, transform=identity,
     return (unravel_collection, last_val) if return_last_val else unravel_collection
 
 
-pytree_metadata = namedtuple('pytree_metadata', ['flat', 'shape', 'size', 'dtype'])
-
-
-def _ravel_list(*leaves):
-    leaves_metadata = tree_map(lambda l: pytree_metadata(
-        jnp.ravel(l), jnp.shape(l), jnp.size(l), canonicalize_dtype(lax.dtype(l))), leaves)
-    leaves_idx = jnp.cumsum(jnp.array((0,) + tuple(d.size for d in leaves_metadata)))
-
-    def unravel_list(arr):
-        return [jnp.reshape(lax.dynamic_slice_in_dim(arr, leaves_idx[i], m.size),
-                            m.shape).astype(m.dtype)
-                for i, m in enumerate(leaves_metadata)]
-
-    flat = jnp.concatenate([m.flat for m in leaves_metadata]) if leaves_metadata else jnp.array([])
-    return flat, unravel_list
-
-
-def ravel_pytree(pytree):
-    leaves, treedef = tree_flatten(pytree)
-    flat, unravel_list = _ravel_list(*leaves)
-
-    def unravel_pytree(arr):
-        return tree_unflatten(treedef, unravel_list(arr))
-
-    return flat, unravel_pytree
-
-
 def soft_vmap(fn, xs, batch_ndims=1, chunk_size=None):
     """
     Vectorizing map that maps a function `fn` over `batch_ndims` leading axes
@@ -293,19 +383,28 @@ def soft_vmap(fn, xs, batch_ndims=1, chunk_size=None):
     # we'll do map(vmap(fn), xs) and make xs.shape = (num_chunks, chunk_size, ...)
     num_chunks = batch_size = int(np.prod(batch_shape))
     prepend_shape = (-1,) if batch_size > 1 else ()
-    xs = tree_map(lambda x: jnp.reshape(x, prepend_shape + jnp.shape(x)[batch_ndims:]), xs)
+    xs = tree_map(
+        lambda x: jnp.reshape(x, prepend_shape + jnp.shape(x)[batch_ndims:]), xs
+    )
     # XXX: probably for the default behavior with chunk_size=None,
     # it is better to catch OOM error and reduce chunk_size by half until OOM disappears.
     chunk_size = batch_size if chunk_size is None else min(batch_size, chunk_size)
     if chunk_size > 1:
         pad = chunk_size - (batch_size % chunk_size)
-        xs = tree_map(lambda x: jnp.pad(x, ((0, pad),) + ((0, 0),) * (np.ndim(x) - 1)), xs)
+        xs = tree_map(
+            lambda x: jnp.pad(x, ((0, pad),) + ((0, 0),) * (np.ndim(x) - 1)), xs
+        )
         num_chunks = batch_size // chunk_size + int(pad > 0)
         prepend_shape = (-1,) if num_chunks > 1 else ()
-        xs = tree_map(lambda x: jnp.reshape(x, prepend_shape + (chunk_size,) + jnp.shape(x)[1:]), xs)
+        xs = tree_map(
+            lambda x: jnp.reshape(x, prepend_shape + (chunk_size,) + jnp.shape(x)[1:]),
+            xs,
+        )
         fn = vmap(fn)
 
     ys = lax.map(fn, xs) if num_chunks > 1 else fn(xs)
     map_ndims = int(num_chunks > 1) + int(chunk_size > 1)
-    ys = tree_map(lambda y: jnp.reshape(y, (-1,) + jnp.shape(y)[map_ndims:])[:batch_size], ys)
+    ys = tree_map(
+        lambda y: jnp.reshape(y, (-1,) + jnp.shape(y)[map_ndims:])[:batch_size], ys
+    )
     return tree_map(lambda y: jnp.reshape(y, batch_shape + jnp.shape(y)[1:]), ys)
