@@ -22,11 +22,15 @@ def model(deterministic=True):
         numpyro.deterministic("x_copy", x)
 
 
-@pytest.mark.parametrize('deterministic', [True, False])
-@pytest.mark.parametrize('find_heuristic_step_size', [True, False])
+@pytest.mark.parametrize("deterministic", [True, False])
+@pytest.mark.parametrize("find_heuristic_step_size", [True, False])
 def test_mcmc_one_chain(deterministic, find_heuristic_step_size):
     GLOBAL["count"] = 0
-    mcmc = MCMC(NUTS(model, find_heuristic_step_size=find_heuristic_step_size), 100, 100)
+    mcmc = MCMC(
+        NUTS(model, find_heuristic_step_size=find_heuristic_step_size),
+        num_warmup=100,
+        num_samples=100,
+    )
     mcmc.run(random.PRNGKey(0), deterministic=deterministic)
     mcmc.get_samples()
 
@@ -37,11 +41,13 @@ def test_mcmc_one_chain(deterministic, find_heuristic_step_size):
         assert GLOBAL["count"] == 3 + num_traces_for_heuristic
 
 
-@pytest.mark.parametrize('deterministic', [True, False])
-@pytest.mark.skipif(xla_bridge.device_count() < 2, reason="only one device is available")
+@pytest.mark.parametrize("deterministic", [True, False])
+@pytest.mark.skipif(
+    xla_bridge.device_count() < 2, reason="only one device is available"
+)
 def test_mcmc_parallel_chain(deterministic):
     GLOBAL["count"] = 0
-    mcmc = MCMC(NUTS(model), 100, 100, num_chains=2)
+    mcmc = MCMC(NUTS(model), num_warmup=100, num_samples=100, num_chains=2)
     mcmc.run(random.PRNGKey(0), deterministic=deterministic)
     mcmc.get_samples()
 
@@ -51,7 +57,7 @@ def test_mcmc_parallel_chain(deterministic):
         assert GLOBAL["count"] == 3
 
 
-@pytest.mark.parametrize('deterministic', [True, False])
+@pytest.mark.parametrize("deterministic", [True, False])
 def test_autoguide(deterministic):
     GLOBAL["count"] = 0
     guide = AutoDiagonalNormal(model)
